import abc


class TermSum(metaclass=abc.ABCMeta):
    def __init__(self):
        self.terms = []

    @property
    @abc.abstractmethod
    def _lefthandside(self): pass  # pragma: no cover

    @property
    @abc.abstractmethod
    def _terms_type(self): pass  # pragma: no cover

    @property
    def _repr(self):
        """Property creating and returning representation string."""
        terms_repr = [term._repr for term in self.terms]
        return " + ".join(terms_repr)

    @property
    def _latex(self):
        s = self._lefthandside

        for term in self.terms:
            if term._latex[1] != "-" and s[-1] != "=":
                s += "+"
            s += term._latex[1:-1]

        if self.terms == []:
            s += "0"
        s += "$"

        return s

    def __repr__(self):
        """A representation method."""
        return self._repr

    def _repr_latex_(self):
        """A LaTeX representation method in Jupyter notebook."""
        return self._latex

    def _add(self, value):
        """Add Term or TermSum to the TermSum"""
        if isinstance(value, self._terms_type):
            setattr(value, "_termsum", self)
            self.terms.append(value)
            setattr(self, value.name, value)
        elif isinstance(value, self.__class__):
            for term in value.terms:
                setattr(term, "_termsum", self)
                self.terms.append(term)
                setattr(self, term.name, term)
        else:
            msg = ("Cannot add type(value)={} "
                   "to {}.".format(value, self.__class__))
            raise TypeError(msg)

    def __iadd__(self, other):
        """Implementation for += operation."""
        self._add(other)
        return self

    @property
    def _script(self):
        """This method should be provided by the specific micromagnetic
        calculator"""
        raise NotImplementedError



class Data:
    def __init__(self, system, cls):
        self.system = system
        self.cls = cls

    @property
    def energy(self):
        raise NotImplementedError

    @property
    def energy_density(self):
        raise NotImplementedError

    @property
    def effective_field(self):
        raise NotImplementedError



import pytest
import micromagneticmodel as mm


class TestEnergyProperties:
    def setup(self):
        self.system = mm.System()
        self.system.hamiltonian = mm.Exchange(A=1)

    def test_data(self):
        assert isinstance(self.system.hamiltonian.exchange._data, mm.Data)
        assert isinstance(self.system.hamiltonian._data, mm.Data)

    def test_energy(self):
        with pytest.raises(NotImplementedError):
            energy = self.system.hamiltonian.energy
        with pytest.raises(NotImplementedError):
            energy = self.system.hamiltonian.exchange.energy

    def test_energy_density(self):
        with pytest.raises(NotImplementedError):
            energy = self.system.hamiltonian.energy_density
        with pytest.raises(NotImplementedError):
            energy = self.system.hamiltonian.exchange.energy_density

    def test_effective_field(self):
        with pytest.raises(NotImplementedError):
            energy = self.system.hamiltonian.effective_field
        with pytest.raises(NotImplementedError):
            energy = self.system.hamiltonian.exchange.effective_field



import importlib


class EnergyProperties:
    @property
    def _data(self):
        module = importlib.__import__(self.__class__.__module__)
        if self.__class__.__name__ == "Hamiltonian":
            system = self._system
        else:
            system = self._termsum._system
        return module.Data(system, self.__class__.__name__)

    @property
    def energy(self):
        return self._data.energy

    @property
    def energy_density(self):
        return self._data.energy_density

    @property
    def effective_field(self):
        return self._data.effective_field



import micromagneticmodel as mm
from .dynamicsterm import DynamicsTerm


class Dynamics(mm.util.TermSum):
    _lefthandside = '$\\frac{\partial \mathbf{m}}{\partial t}='
    _terms_type = DynamicsTerm



import pytest
import numbers
import micromagneticmodel as mm


class TestDMI:
    def setup(self):
        self.valid_args = [-1, 2.0, 5e-11, -1e-12, 1e-13, 1e-14, -1e6]
        self.invalid_args = ['a', (1, 2), '0', [1, 2, 3]]

    def test_init_valid_args(self):
        for D in self.valid_args:
            dmi = mm.DMI(D)
            assert dmi.D == D
            assert isinstance(dmi.D, numbers.Real)

    def test_init_invalid_args(self):
        for D in self.invalid_args:
            with pytest.raises(Exception):
                dmi = mm.DMI(D)

    def test_repr_latex_(self):
        for D in self.valid_args:
            dmi = mm.DMI(D, crystalclass="T")
            latex = dmi._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == "$"
            assert latex[-1] == "$"
            assert 'D' in latex
            assert latex.count('\mathbf{m}') == 2

            dmi = mm.DMI(D, crystalclass="cnv")
            latex = dmi._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == latex[-1] == '$'
            assert '\\nabla' in latex
            assert 'D' in latex
            assert latex.count('\\frac') == 0

            dmi = mm.DMI(D, crystalclass="d2d")
            latex = dmi._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == latex[-1] == '$'
            assert '\\partial' in latex
            assert '\\hat' in latex
            assert 'D' in latex
            assert latex.count('\\frac') == 2

    def test_name(self):
        for D in self.valid_args:
            dmi = mm.DMI(D)
            assert dmi.name == 'dmi'

    def test_repr(self):
        for D in self.valid_args:
            dmi = mm.DMI(D, crystalclass="t")
            assert repr(dmi) == ('DMI(D={}, crystalclass=\"t\", '
                                 'name=\"{}\")').format(D, "dmi")

            dmi = mm.DMI(D, crystalclass="d2d", name="test_name")
            assert repr(dmi) == ("DMI(D={}, crystalclass=\"d2d\", "
                                 "name=\"test_name\")").format(D)

    def test_script(self):
        for D in self.valid_args:
            dmi = mm.DMI(D)
            with pytest.raises(NotImplementedError):
                script = dmi._script



import pytest
import numpy as np
import discretisedfield as df
import micromagneticmodel as mm


class TestSystem:
    def setup(self):
        p1 = (0, 0, 0)
        p2 = (1., 1., 1.)
        cell = (0.2, 0.2, 0.2)
        self.mesh = df.Mesh(p1, p2, cell)

    def test_init(self):
        system = mm.System(name="test_sim")

        assert isinstance(system.hamiltonian, mm.Hamiltonian)
        assert system.hamiltonian.terms == []

        system.hamiltonian = mm.Exchange(1e-12) + mm.Demag()
        assert len(system.hamiltonian.terms)

        assert isinstance(system.dynamics, mm.Dynamics)
        assert system.dynamics.terms == []

        assert system.name == "test_sim"

    def test_wrong_kwargs(self):
        with pytest.raises(AttributeError):
            system = mm.System(name="wrong_mesh", attr="a")

    def test_set_m(self):
        system = mm.System(name="test_sim")

        system.m = df.Field(self.mesh, dim=3, value=(0, 0, 1))

        assert isinstance(system.m, df.Field)
        assert len(system.m.average) == 3

    def test_set_wrong_m(self):
        system = mm.System(name="test_sim")

        with pytest.raises(TypeError):
            system.m = "a"

    def test_set_hamiltonian_with_energyterm(self):
        system = mm.System()
        assert isinstance(system.hamiltonian, mm.Hamiltonian)
        system.hamiltonian = mm.Exchange(1e-12)
        assert isinstance(system.hamiltonian, mm.Hamiltonian)
        assert len(system.hamiltonian.terms) == 1

    def test_set_hamiltonian_with_zero(self):
        system = mm.System()
        system.hamiltonian = mm.Exchange(1e-12) + mm.Demag()
        assert isinstance(system.hamiltonian, mm.Hamiltonian)
        assert len(system.hamiltonian.terms) == 2

        system.hamiltonian = 0
        assert isinstance(system.hamiltonian, mm.Hamiltonian)
        assert len(system.hamiltonian.terms) == 0

    def test_set_hamiltonian_wrong(self):
        system = mm.System()
        assert isinstance(system.hamiltonian, mm.Hamiltonian)
        with pytest.raises(TypeError):
            system.hamiltonian = 1

    def test_set_dynamics_with_dynamicsterm(self):
        system = mm.System()
        assert isinstance(system.dynamics, mm.Dynamics)
        system.dynamics = mm.Precession(2.211e5)
        assert isinstance(system.dynamics, mm.Dynamics)
        assert len(system.dynamics.terms) == 1

    def test_set_dynamics_with_zero(self):
        system = mm.System()
        system.dynamics = mm.Precession(2e5) + mm.Damping(0.1)
        assert isinstance(system.dynamics, mm.Dynamics)
        assert len(system.dynamics.terms) == 2

        system.dynamics = 0
        assert isinstance(system.dynamics, mm.Dynamics)
        assert len(system.dynamics.terms) == 0

    def test_set_dynamics_wrong(self):
        system = mm.System()
        assert isinstance(system.dynamics, mm.Dynamics)
        with pytest.raises(TypeError):
            system.dynamics = 1

    def test_script(self):
        system = mm.System(name="test_sim")

        with pytest.raises(NotImplementedError):
            script = system._script

    def test_repr(self):
        # 'empty' system object
        system = mm.System()
        r = repr(system)
        assert 'hamiltonian' in r
        assert 'dynamics' in r
        assert 'System' in r

        # system object with name
        system = mm.System(name='testname')
        r = repr(system)
        assert 'testname' in r



from .test_exchange import TestExchange
from .test_zeeman import TestZeeman
from .test_uniaxialanisotropy import TestUniaxialAnisotropy
from .test_cubicanisotropy import TestCubicAnisotropy
from .test_demag import TestDemag
from .test_dmi import TestDMI
from .test_hamiltonian import TestHamiltonian
from .test_precession import TestPrecession
from .test_damping import TestDamping
from .test_stt import TestSTT
from .test_dynamics import TestDynamics
from .test_driver import TestDriver
from .test_mindriver import TestMinDriver
from .test_timedriver import TestTimeDriver
from .test_system import TestSystem
from .test_consts import TestConsts



import pytest
import micromagneticmodel as mm


class TestData:
    def setup(self):
        self.data = mm.Data(system=None, cls=None)

    def test_energy(self):
        with pytest.raises(NotImplementedError):
            energy = self.data.energy

    def test_energy_density(self):
        with pytest.raises(NotImplementedError):
            energy = self.data.energy_density

    def test_effective_field(self):
        with pytest.raises(NotImplementedError):
            energy = self.data.effective_field



import pytest
import micromagneticmodel as mm


class TestHysteresisDriver:
    def setup(self):
        self.hd = mm.HysteresisDriver(Hmin=(0, 0, 0), Hmax=(1, 1, 1), n=100)

    def test_init(self):
        assert self.hd.Hmin == (0, 0, 0)
        assert self.hd.Hmax == (1, 1, 1)
        assert self.hd.n == 100



import pytest
import numbers
import micromagneticmodel as mm


class TestSTT:
    def setup(self):
        self.valid_args = [((1, -2, 3), 1),
                           ((-1.0, 0, 1e-6), 2.0),
                           ((0, 0, 0), 5e-11),
                           ((11, 2, 19), -1e-12)]
        self.invalid_args = [((1, -2), 1),
                             ((-1.0, 0, 1e-6), "2.0"),
                             ((0, 0, 0, 9), 5e-11),
                             ((11, 2, 19), -1e-12+2j)]

    def test_init_valid_args(self):
        for arg in self.valid_args:
            u, beta = arg
            stt = mm.STT(u, beta)

            assert stt.u == u
            assert isinstance(stt.u, tuple)
            assert stt.beta == beta
            assert isinstance(stt.beta, numbers.Real)

    def test_init_invalid_args(self):
        for arg in self.invalid_args:
            u, beta = arg

            with pytest.raises(Exception):
                stt = mm.STT(gamma)

    def test_repr_latex_(self):
        for arg in self.valid_args:
            u, beta = arg
            stt = mm.STT(u, beta)
            latex = stt._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == latex[-1] == '$'
            assert '\\beta' in latex
            assert '\mathbf{m}' in latex
            assert '\mathbf{u}' in latex
            assert '\\times' in latex
            assert '\\boldsymbol' in latex
            assert '\\nabla' in latex

    def test_name(self):
        for arg in self.valid_args:
            u, beta = arg
            stt = mm.STT(u, beta)

            assert stt.name == 'stt'

    def test_repr(self):
        for arg in self.valid_args:
            u, beta = arg
            stt = mm.STT(u, beta)

            assert repr(stt) == ("STT(u={}, beta={}, "
                                 "name=\"stt\")".format(u, beta))

        stt = mm.STT((1, 2, 3), 15, name="test_name")
        assert repr(stt) == 'STT(u=(1, 2, 3), beta=15, name=\"test_name\")'

    def test_script(self):
        for arg in self.valid_args:
            u, beta = arg
            stt = mm.STT(u, beta)
            with pytest.raises(NotImplementedError):
                script = stt._script



from .energyproperties import EnergyProperties
from .energyterm import EnergyTerm
from .exchange import Exchange
from .zeeman import Zeeman
from .demag import Demag
from .dmi import DMI
from .uniaxialanisotropy import UniaxialAnisotropy
from .cubicanisotropy import CubicAnisotropy
from .hamiltonian import Hamiltonian



import pytest
import micromagneticmodel as mm


class TestDynamics:
    def setup(self):
        gamma = 2.21e5
        self.precession = mm.Precession(gamma)
        alpha = 0.5
        self.damping = mm.Damping(alpha)
        u = (0, 0, 500)
        beta = 0.2
        self.stt = mm.STT(u=u, beta=beta)

        self.terms = [self.precession,
                      self.damping,
                      self.stt]

        self.invalid_terms = [1, 2.5, 0, 'abc', [3, 7e-12],
                              [self.precession, self.damping]]

    def test_add_terms(self):
        dynamics = mm.Dynamics()
        for term in self.terms:
            dynamics._add(term)
            assert isinstance(dynamics, mm.Dynamics)
            assert isinstance(dynamics.terms, list)
            assert dynamics.terms[-1] == term
            assert dynamics.terms[-1].name == term.name

        assert len(dynamics.terms) == 3

    def test_add_sum_of_terms(self):
        dynamics = self.precession + self.damping + self.stt

        assert isinstance(dynamics, mm.Dynamics)
        assert isinstance(dynamics.terms, list)
        assert len(dynamics.terms) == 3

    def test_add_dynamics(self):
        term_sum = self.precession + self.damping + self.stt
        dynamics = mm.Dynamics()
        dynamics += term_sum

        assert len(dynamics.terms) == 3

    def test_iadd(self):
        dynamics = mm.Dynamics()
        for term in self.terms:
            dynamics += term

            assert isinstance(dynamics, mm.Dynamics)
            assert isinstance(dynamics.terms, list)
            assert dynamics.terms[-1] == term
            assert dynamics.terms[-1].name == term.name

        assert len(dynamics.terms) == 3

    def test_repr_latex(self):
        dynamics = mm.Dynamics()
        latex = dynamics._repr_latex_()
        assert latex[0] == latex[-1] == '$'
        assert latex.count('$') == 2
        assert '\\frac' in latex
        assert latex[-2] == '0'

        for term in self.terms:
            dynamics._add(term)

        latex = dynamics._repr_latex_()

        assert latex[0] == latex[-1] == '$'
        assert latex.count('$') == 2
        assert '-\gamma_{0}^{*}' in latex
        assert '\mathbf{m}' in latex
        assert '\mathbf{H}_\\text{eff}' in latex
        assert '\\times' in latex
        assert '\\alpha' in latex
        assert latex.count('-') == 2
        assert latex.count('+') == 2
        assert latex.count('=') == 1
        assert latex.count('\partial') == 4

    def test_add_exception(self):
        dynamics = mm.Dynamics()
        for term in self.invalid_terms:
            with pytest.raises(TypeError):
                dynamics += term

    def test_repr(self):
        dynamics = self.precession + self.damping + self.stt

        exp_str = ("Precession(gamma=221000.0, name=\"precession\") + "
                   "Damping(alpha=0.5, name=\"damping\") + "
                   "STT(u=(0, 0, 500), beta=0.2, name=\"stt\")")
        assert repr(dynamics) == exp_str

    def test_getattr(self):
        dynamics = self.precession + self.damping + self.stt

        assert isinstance(dynamics.precession, mm.Precession)
        assert dynamics.precession.gamma == 2.21e5

        assert isinstance(dynamics.damping, mm.Damping)
        assert dynamics.damping.alpha == 0.5

        assert isinstance(dynamics.stt, mm.STT)
        assert dynamics.stt.u == (0, 0, 500)
        assert dynamics.stt.beta == 0.2

    def test_getattr_error(self):
        dynamics = self.precession + self.damping

        with pytest.raises(AttributeError):
            stt = dynamics.stt

    def test_script(self):
        dynamics = mm.Dynamics()
        with pytest.raises(NotImplementedError):
            script = dynamics._script



import joommfutil.typesystem as ts
from .dynamicsterm import DynamicsTerm


@ts.typesystem(alpha=ts.Scalar(unsigned=True),
               name=ts.Name(const=True))
class Damping(DynamicsTerm):
    _latex = ("$\\alpha \mathbf{m} \\times"
              "\\frac{\partial \mathbf{m}}{\partial t}$")

    def __init__(self, alpha, name="damping"):
        """A damping dynamics term class.

        Args:
            alpha (Real): Gilbert damping

        """
        self.alpha = alpha
        self.name = name

    @property
    def _repr(self):
        """A representation string property.

        Returns:
           A representation string.

        """
        return "Damping(alpha={}, name=\"{}\")".format(self.alpha, self.name)



import joommfutil.typesystem as ts
from .dynamicsterm import DynamicsTerm


@ts.typesystem(gamma=ts.Scalar(unsigned=True),
               name=ts.Name(const=True))
class Precession(DynamicsTerm):
    _latex = ("$-\gamma_{0}^{*} \mathbf{m} \\times "
              "\mathbf{H}_\\text{eff}$")

    def __init__(self, gamma, name="precession"):
        """A precession dynamics term class.

        Args:
            gamma (Real): gyrotropic ratio (m/As)

        """
        self.gamma = gamma
        self.name = name

    @property
    def _repr(self):
        """A representation string property.

        Returns:
           A representation string.

        """
        return "Precession(gamma={}, name=\"{}\")".format(self.gamma, self.name)



import pytest
import numbers
import numpy as np
import micromagneticmodel as mm


class TestUniaxialAnisotropy:
    def setup(self):
        self.valid_args = [(1, 5, (0, 1, 0)),
                           (5e6, 0, [1, 1, 1]),
                           (-25.6e-3, -2e-9, np.array([0, 0, 1])),
                           (1.5, 0, [1e6, 1e6, 5e9])]
        self.invalid_args = [('1', 5e6, (0, 1, 0)),
                             (5e6, 1e6, '(1, 1, 1)'),
                             (1e-3, 11, (0, 0, 0, 1)),
                             (5, 3.14, 5.0),
                             (-7e3, 2.7e4, ('1', 2e6, 0)),
                             ((1, 0, 0), 1e9, (0, 0, 1)),
                             (1, (5, 0), (0, 1, 0))]

    def test_init_valid_args(self):
        for K1, K2, u in self.valid_args:
            anisotropy = mm.UniaxialAnisotropy(K1=K1, K2=K2, u=u)
            assert anisotropy.K1 == K1
            assert anisotropy.K2 == K2
            assert isinstance(anisotropy.K1, numbers.Real)
            assert isinstance(anisotropy.K2, numbers.Real)
            assert isinstance(anisotropy.u, (tuple, list, np.ndarray))
            assert len(anisotropy.u) == 3
            assert all([isinstance(i, numbers.Real) for i in anisotropy.u])

    def test_init_invalid_args(self):
        for K1, K2, u in self.invalid_args:
            with pytest.raises(Exception):
                anisotropy = mm.UniaxialAnisotropy(K1=K1, K2=K2, u=u)

    def test_repr_latex(self):
        for K1, K2, u in self.valid_args:
            anisotropy = mm.UniaxialAnisotropy(K1=K1, K2=K2, u=u)
            latex = anisotropy._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == latex[-1] == '$'
            assert 'K_{1}' in latex
            assert '\mathbf{u}' in latex
            assert '\mathbf{m}' in latex
            assert '^{2}' in latex
            assert '\cdot' in latex
            if K2 != 0:
                assert 'K_{2}' in latex
                assert '^{4}' in latex

    def test_name(self):
        for K1, K2, u in self.valid_args:
            anisotropy = mm.UniaxialAnisotropy(K1=K1, K2=K2, u=u)
            assert anisotropy.name == 'uniaxialanisotropy'

    def test_repr(self):
        for K1, K2, u in self.valid_args:
            anisotropy = mm.UniaxialAnisotropy(K1=K1, K2=K2, u=u)
            exp_str = ("UniaxialAnisotropy(K1={}, K2={}, u={}, "
                       "name=\"{}\")").format(K1, K2, u, "uniaxialanisotropy")
            assert repr(anisotropy) == exp_str

        anisotropy = mm.UniaxialAnisotropy(1000, (0, 0, 1), name="test_name")
        assert repr(anisotropy) == ("UniaxialAnisotropy(K1=1000, K2=0, "
                                    "u=(0, 0, 1), name=\"test_name\")")

    def test_script(self):
        for K1, K2, u in self.valid_args:
            anisotropy = mm.UniaxialAnisotropy(K1=K1, K2=K2, u=u)
            with pytest.raises(NotImplementedError):
                script = anisotropy._script



import pytest
import numbers
import micromagneticmodel as mm


class TestDamping:
    def setup(self):
        self.valid_args = [1, 2.0, 5e-11, 1e-12, 1e-13, 1e-14, 1e6]
        self.invalid_args = [-1, -2.1, 'a', (1, 2), -3.6e-6, '0', [1, 2, 3]]

    def test_init_valid_args(self):
        for alpha in self.valid_args:
            damping = mm.Damping(alpha)

            assert damping.alpha == alpha
            assert isinstance(damping.alpha, numbers.Real)

    def test_init_invalid_args(self):
        for alpha in self.invalid_args:
            with pytest.raises(Exception):
                damping = mm.Damping(alpha)

    def test_repr_latex_(self):
        for alpha in self.valid_args:
            damping = mm.Damping(alpha)
            latex = damping._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == latex[-1] == '$'
            assert '\\alpha' in latex
            assert '\mathbf{m}' in latex
            assert '\\frac' in latex
            assert '\\times' in latex
            assert latex.count('\partial') == 2

    def test_name(self):
        for alpha in self.valid_args:
            damping = mm.Damping(alpha)
            assert damping.name == 'damping'

    def test_repr(self):
        for alpha in self.valid_args:
            damping = mm.Damping(alpha)
            assert repr(damping) == ('Damping(alpha={}, '
                                     'name=\"damping\")').format(alpha)

        damping = mm.Damping(alpha=0.05, name="test_name")
        assert repr(damping) == 'Damping(alpha=0.05, name=\"test_name\")'

    def test_script(self):
        for alpha in self.valid_args:
            damping = mm.Damping(alpha)
            with pytest.raises(NotImplementedError):
                script = damping._script



import pytest
import numbers
import micromagneticmodel as mm


class TestExchange:
    def setup(self):
        self.valid_args = [1, 2.0, 5e-11, 1e-12, 1e-13, 1e-14, 1e6]
        self.invalid_args = [-1, -2.1, 'a', (1, 2), -3.6e-6, '0', [1, 2, 3]]

    def test_init_valid_args(self):
        for A in self.valid_args:
            exchange = mm.Exchange(A)
            assert exchange.A == A
            assert isinstance(exchange.A, numbers.Real)

    def test_init_invalid_args(self):
        for A in self.invalid_args:
            with pytest.raises(Exception):
                exchange = mm.Exchange(A)

    def test_repr_latex_(self):
        for A in self.valid_args:
            exchange = mm.Exchange(A)
            latex = exchange._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == latex[-1] == '$'
            assert latex.count('\\nabla') == 1
            assert 'A' in latex
            assert '\mathbf{m}' in latex
            assert latex.count('+') == 0

    def test_name(self):
        for A in self.valid_args:
            exchange = mm.Exchange(A)
            assert exchange.name == 'exchange'

    def test_repr(self):
        for A in self.valid_args:
            exchange = mm.Exchange(A)
            assert repr(exchange) == ('Exchange(A={}, '
                                      'name=\"{}\")').format(A, "exchange")

        exchange = mm.Exchange(8.78e-12, name="test_name")
        assert repr(exchange) == "Exchange(A=8.78e-12, name=\"test_name\")"

    def test_script(self):
        for A in self.valid_args:
            exchange = mm.Exchange(A)
            with pytest.raises(NotImplementedError):
                script = exchange._script



class Driver:
    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    def drive(self):
        raise NotImplementedError

    @property
    def _script(self):
        raise NotImplementedError



import pytest
import micromagneticmodel as mm


class TestTimeDriver:
    def setup(self):
        self.td = mm.TimeDriver(t=1e-9, n=100)

    def test_init(self):
        assert self.td.t == 1e-9
        assert self.td.n == 100



import pytest
import numbers
import numpy as np
import micromagneticmodel as mm


class TestZeeman:
    def setup(self):
        self.valid_args = [(1, 1.4, 1),
                           (0, 0, 1),
                           [1.2, 0, 0],
                           (0.56e6, 1.98e6, -1.1e7),
                           np.array([15e6, 0, 5e-8])]
        self.invalid_args = [(1, 1),
                             1,
                             (1.2, 0, 0, 5),
                             (0.56, 1.98, '-1.1'),
                             ([15], [0], [np.pi])]

    def test_init_valid_args(self):
        for H in self.valid_args:
            zeeman = mm.Zeeman(H)
            assert isinstance(zeeman.H, (tuple, list, np.ndarray))
            assert len(zeeman.H) == 3
            assert all([isinstance(i, numbers.Real) for i in zeeman.H])

    def test_init_invalid_args(self):
        for H in self.invalid_args:
            with pytest.raises(Exception):
                zeeman = mm.Zeeman(H)

    def test_repr_latex(self):
        for H in self.valid_args:
            zeeman = mm.Zeeman(H)
            latex = zeeman._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert latex[0] == latex[-1] == '$'
            assert latex[1] == '-'
            assert '\\mu_{0}' in latex
            assert '\mathbf{H}' in latex
            assert '\mathbf{m}' in latex
            assert '\cdot' in latex
            assert 'M_\\text{s}' in latex

    def test_name(self):
        for H in self.valid_args:
            zeeman = mm.Zeeman(H)
            assert zeeman.name == 'zeeman'

    def test_repr(self):
        for H in self.valid_args:
            zeeman = mm.Zeeman(H)
            assert repr(zeeman) == ('Zeeman(H={}, '
                                    'name=\"{}\")'.format(H, "zeeman"))

        zeeman = mm.Zeeman(H=(1, 0, 1), name="test_name")
        assert repr(zeeman) == "Zeeman(H=(1, 0, 1), name=\"test_name\")"

    def test_script(self):
        for H in self.valid_args:
            zeeman = mm.Zeeman(H)
            with pytest.raises(NotImplementedError):
                script = zeeman._script



import joommfutil.typesystem as ts
from .energyterm import EnergyTerm


@ts.typesystem(K1=ts.Scalar,
               K2=ts.Scalar,
               u=ts.Vector(size=3),
               name=ts.Name(const=True))
class UniaxialAnisotropy(EnergyTerm):
    def __init__(self, K1, u, K2=0, name="uniaxialanisotropy"):
        """Uniaxial anisotropy energy abstract class.

        Parameters
        ----------
        K1, K2 : Real
            Uniaxial anisotropy energy constant (J/m**3)
        u : (3,) array_like
            Uniaxial anisotropy axis

        Examples
        --------
        Creating a uniaxial anisotropy object

        >>> import micromagneticmodel as mm
        >>> ua = mm.UniaxialAnisotropy(K1=5e6, K2=1e3, u=(0, 0, 1))

        """
        self.K1 = K1
        self.K2 = K2
        self.u = u
        self.name = name

    @property
    def _latex(self):
        first_term = "-K_{1} (\mathbf{m} \cdot \mathbf{u})^{2}"
        second_term = "-K_{2} (\mathbf{m} \cdot \mathbf{u})^{4}"
        if self.K2 == 0:
            return "${}$".format(first_term)
        else:
            return "${}$".format(first_term+second_term)

    @property
    def _repr(self):
        """A representation string property.

        Returns
        -------
        str
            A representation string.

        """
        return ("UniaxialAnisotropy(K1={}, K2={}, u={}, "
                "name=\"{}\")").format(self.K1, self.K2, self.u, self.name)



import joommfutil.typesystem as ts
from .energyterm import EnergyTerm


@ts.typesystem(name=ts.Name(const=True))
class Demag(EnergyTerm):
    _latex = ("$-\\frac{1}{2}\mu_{0}M_\\text{s}"
              "\mathbf{m} \cdot \mathbf{H}_\\text{d}$")

    def __init__(self, name="demag"):
        """Abstract demagnetisation energy class."""
        self.name = name

    @property
    def _repr(self):
        """A representation string property.

        Returns:
           A representation string.

        """
        return "Demag(name=\"{}\")".format(self.name)



import joommfutil.typesystem as ts
from .energyterm import EnergyTerm


@ts.typesystem(K1=ts.Scalar,
               u1=ts.Vector(size=3),
               u2=ts.Vector(size=3),
               name=ts.Name(const=True))
class CubicAnisotropy(EnergyTerm):
    def __init__(self, K1, u1, u2, name="cubicanisotropy"):
        """Cubic anisotropy energy abstract class.

        Parameters
        ----------
        K1 : Real
            Cubic anisotropy energy constant (J/m**3)
        u1, u2 : (3,) array_like
            Cubic anisotropy axes

        """
        self.K1 = K1
        self.u1 = u1
        self.u2 = u2
        self.name = name

    @property
    def _latex(self):
        a1 = "(\mathbf{m} \cdot \mathbf{u}_{1})^{2}"
        a2 = "(\mathbf{m} \cdot \mathbf{u}_{2})^{2}"
        a3 = "(\mathbf{m} \cdot \mathbf{u}_{3})^{2}"
        return "$-K_{{1}} [{0}{1}+{1}{2}+{2}{0}]$".format(a1, a2, a3)

    @property
    def _repr(self):
        """A representation string property.

        Returns
        -------
        str
            A representation string.

        """
        return ("CubicAnisotropy(K1={}, u1={}, u2={}, "
                "name=\"{}\")").format(self.K1, self.u1, self.u2, self.name)



import pytest
import numbers
import micromagneticmodel as mm


class TestPrecession:
    def setup(self):
        self.valid_args = [1, 2.0, 5e-11, 1e-12, 1e-13, 1e-14, 1e6]
        self.invalid_args = [-1, -2.1, 'a', (1, 2), -3.6e-6, '0', [1, 2, 3]]

    def test_init_valid_args(self):
        for gamma in self.valid_args:
            precession = mm.Precession(gamma)
            assert precession.gamma == gamma
            assert isinstance(precession.gamma, numbers.Real)

    def test_init_invalid_args(self):
        for gamma in self.invalid_args:
            with pytest.raises(Exception):
                precession = mm.Precession(gamma)

    def test_repr_latex_(self):
        for gamma in self.valid_args:
            precession = mm.Precession(gamma)
            latex = precession._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == latex[-1] == '$'
            assert '\gamma' in latex
            assert '^{*}' in latex
            assert '_{0}' in latex
            assert '\mathbf{m}' in latex
            assert '\mathbf{H}_\\text{eff}' in latex
            assert '\\times' in latex

    def test_name(self):
        for gamma in self.valid_args:
            precession = mm.Precession(gamma)
            assert precession.name == 'precession'

    def test_repr(self):
        for gamma in self.valid_args:
            precession = mm.Precession(gamma)
            assert repr(precession) == ('Precession(gamma={}, '
                                        'name=\"{}\")').format(gamma,
                                                               "precession")

        precession = mm.Precession(gamma=2.211e5, name="test_name")
        assert repr(precession) == ('Precession(gamma=221100.0, '
                                    'name=\"test_name\")')

    def test_script(self):
        for gamma in self.valid_args:
            precession = mm.Precession(gamma)
            with pytest.raises(NotImplementedError):
                script = precession._script



import joommfutil.typesystem as ts
from .energyterm import EnergyTerm


@ts.typesystem(A=ts.Scalar(unsigned=True),
               name=ts.Name(const=True))
class Exchange(EnergyTerm):
    _latex = "$A (\\nabla \mathbf{m})^{2}$"

    def __init__(self, A, name="exchange"):
        """An exchange energy class.

        Args:
            A (Real): exchange energy constant (J/m)

        """
        self.A = A
        self.name = name

    @property
    def _repr(self):
        """A representation string property.

        Returns:
           A representation string.

        """
        return "Exchange(A={}, name=\"{}\")".format(self.A, self.name)



import pytest
import micromagneticmodel as mm


class TestDriver:
    def setup(self):
        self.driver = mm.Driver(a=1, b=2, c='c')

    def test_init(self):
        assert self.driver.a == 1
        assert self.driver.b == 2
        assert self.driver.c == "c"

    def test_drive(self):
        with pytest.raises(NotImplementedError):
            self.driver.drive()

    def test_script(self):
        with pytest.raises(NotImplementedError):
            script = self.driver._script



import micromagneticmodel as mm

def test_version():
    assert isinstance(mm.__version__, str)
    assert '.' in mm.__version__

def test_dependencies():
    assert isinstance(mm.__dependencies__, list)
    assert len(mm.__dependencies__) > 0
    



import micromagneticmodel as mm
from .energyproperties import EnergyProperties
from .energyterm import EnergyTerm


class Hamiltonian(mm.util.TermSum, EnergyProperties):
    _lefthandside = '$\\mathcal{H}='
    _terms_type = EnergyTerm



import pytest
import micromagneticmodel as mm


class TestDemag:
    def test_repr_latex_(self):
        demag = mm.Demag()
        latex = demag._repr_latex_()

        # Assert some characteristics of LaTeX string.
        assert isinstance(latex, str)
        assert latex[0] == latex[-1] == '$'
        assert '\\mu_{0}' in latex
        assert '\mathbf{H}_\\text{d}' in latex
        assert '\mathbf{m}' in latex
        assert '\cdot' in latex
        assert 'M_\\text{s}' in latex
        assert '\\frac{1}{2}' in latex

    def test_name(self):
        demag = mm.Demag()
        assert demag.name == 'demag'

    def test_repr(self):
        demag = mm.Demag()
        assert repr(demag) == 'Demag(name=\"demag\")'

    def test_script(self):
        demag = mm.Demag()
        with pytest.raises(NotImplementedError):
            script = demag._script



import importlib
import discretisedfield as df
import micromagneticmodel as mm
import joommfutil.typesystem as ts


@ts.typesystem(name=ts.Name(const=True))
class System:
    _attributes = ["hamiltonian", "dynamics", "m", "name"]

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if key in System._attributes:
                setattr(self, key, value)
            else:
                raise AttributeError("Unexpected kwarg {}.".format(key))

        self._module = importlib.__import__(self.__class__.__module__)
        if "hamiltonian" not in self.__dict__:
            self.hamiltonian = 0
        if "dynamics" not in self.__dict__:
            self.dynamics = 0

        self.run_number = 0

    @property
    def hamiltonian(self):
        return self._hamiltonian

    @hamiltonian.setter
    def hamiltonian(self, value):
        self._hamiltonian = self._module.Hamiltonian()
        setattr(self._hamiltonian, "_system", self)
        if value == 0:
            pass
        elif isinstance(value, (mm.EnergyTerm, mm.Hamiltonian)):
            self._hamiltonian += value
        else:
            raise TypeError("Unsupported type(value)={}".format(type(value)))

    @property
    def dynamics(self):
        return self._dynamics

    @dynamics.setter
    def dynamics(self, value):
        self._dynamics = self._module.Dynamics()
        setattr(self._dynamics, "_system", self)
        if value == 0:
            pass
        elif isinstance(value, (mm.DynamicsTerm, mm.Dynamics)):
            self._dynamics += value
        else:
            raise TypeError("Unsupported type(value)={}".format(type(value)))

    @property
    def m(self):
        return self._m

    @m.setter
    def m(self, value):
        if isinstance(value, df.Field):
            self._m = value
        else:
            raise TypeError("Unsupported type(m)={}".format(type(value)))

    @property
    def _script(self):
        raise NotImplementedError

    def __repr__(self):
        r = ["System object '{}':".format(self.name)]
        for attribute in System._attributes:
            if attribute == 'name':
                continue
            r.append("\t{:11}: {}".format(attribute,
                                          getattr(self, attribute, "")))
        return "\n".join(r)



import importlib
import numpy as np
import micromagneticmodel as mm


class TestConsts:
    def test_mu0(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert self.module.mu0 == 4*np.pi*1e-7

    def test_e(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert self.module.e == 1.6021766208e-19

    def test_me(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert self.module.me == 9.1093835611e-31

    def test_kB(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert self.module.kB == 1.3806485279e-23

    def test_h(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert self.module.h == 6.62607004081e-34

    def test_g(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert self.module.g == 2.00231930436182

    def test_hbar(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert abs(self.module.hbar - 1.05457180013e-34) < 1e-40

    def test_gamma(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert abs(self.module.gamma - 176085964286.56906) < 1e-3

    def test_muB(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert abs(self.module.muB == 9.27400968e-24) < 1e-30

    def test_gamma0(self):
        self.module = importlib.__import__(self.__class__.__module__)
        assert abs(self.module.gamma0 - 2.2127614872e5) < 1e-3



import pytest
import micromagneticmodel as mm


class TestHamiltonian:
    def setup(self):
        A = 1e-12
        self.exchange = mm.Exchange(A=A)
        H = (0, 0, 1.2e6)
        self.zeeman = mm.Zeeman(H=H)
        K1 = 1e4
        K2 = 3e2
        u = (0, 1, 0)
        self.uniaxialanisotropy = mm.UniaxialAnisotropy(K1=K1, K2=K2, u=u)
        self.demag = mm.Demag()
        D = 1e-3
        crystalclass = "t"
        self.dmi = mm.DMI(D=D, crystalclass=crystalclass)
        K1 = 5e6
        u1 = (0, 0, 1)
        u2 = (0, 1, 0)
        self.cubicanisotropy = mm.CubicAnisotropy(K1=K1, u1=u1, u2=u2)

        self.terms = [self.exchange,
                      self.zeeman,
                      self.uniaxialanisotropy,
                      self.demag,
                      self.dmi,
                      self.cubicanisotropy]

        self.invalid_terms = [1, 2.5, 0, 'abc', [3, 7e-12],
                              [self.exchange, self.zeeman]]

    def test_add_terms(self):
        hamiltonian = mm.Hamiltonian()
        for term in self.terms:
            hamiltonian._add(term)

            assert isinstance(hamiltonian, mm.Hamiltonian)
            assert isinstance(hamiltonian.terms, list)
            assert hamiltonian.terms[-1] == term
            assert hamiltonian.terms[-1].name == term.name

        assert len(hamiltonian.terms) == 6

    def test_add_sum_of_terms(self):
        hamiltonian = self.exchange + self.zeeman + \
                      self.uniaxialanisotropy + self.demag + \
                      self.dmi + self.cubicanisotropy

        assert isinstance(hamiltonian, mm.Hamiltonian)
        assert isinstance(hamiltonian.terms, list)
        assert len(hamiltonian.terms) == 6

    def test_add_hamiltonian(self):
        term_sum = self.exchange + self.zeeman
        hamiltonian = mm.Hamiltonian()
        hamiltonian += term_sum

        assert len(hamiltonian.terms) == 2

    def test_iadd(self):
        hamiltonian = mm.Hamiltonian()
        for term in self.terms:
            hamiltonian += term

            assert isinstance(hamiltonian, mm.Hamiltonian)
            assert isinstance(hamiltonian.terms, list)
            assert hamiltonian.terms[-1] == term
            assert hamiltonian.terms[-1].name == term.name

        assert len(hamiltonian.terms) == 6

    def test_repr_latex(self):
        hamiltonian = mm.Hamiltonian()
        latex = hamiltonian._repr_latex_()
        assert latex[0] == latex[-1] == '$'
        assert latex.count('$') == 2
        assert '\\mathcal{H}' in latex
        assert latex[-2] == '0'

        for term in self.terms:
            hamiltonian._add(term)

        latex = hamiltonian._repr_latex_()

        assert latex[0] == latex[-1] == '$'
        assert latex.count('$') == 2
        assert '\\mathcal{H}=' in latex
        assert 'A' in latex
        assert '\mathbf{m}' in latex
        assert '\mathbf{H}' in latex
        assert '\mathbf{u}' in latex
        assert 'K' in latex
        assert '\mathbf{H}_\\text{d}' in latex
        assert '\cdot' in latex
        assert '\\frac{1}{2}' in latex
        assert 'M_\\text{s}' in latex
        assert latex.count('-') == 5
        assert latex.count('+') == 3
        assert latex.count('=') == 1
        assert latex.count('\\nabla') == 2
        assert latex.count('\\times') == 1

    def test_add_exception(self):
        hamiltonian = mm.Hamiltonian()
        for term in self.invalid_terms:
            with pytest.raises(TypeError):
                hamiltonian += term

    def test_repr(self):
        hamiltonian = self.exchange + self.zeeman + \
                      self.uniaxialanisotropy + self.demag + \
                      self.dmi + self.cubicanisotropy

        exp_str = ("Exchange(A=1e-12, name=\"exchange\") + "
                   "Zeeman(H=(0, 0, 1200000.0), name=\"zeeman\") + "
                   "UniaxialAnisotropy(K1=10000.0, K2=300.0, u=(0, 1, 0), "
                   "name=\"uniaxialanisotropy\") + "
                   "Demag(name=\"demag\") + "
                   "DMI(D=0.001, crystalclass=\"t\", name=\"dmi\") + "
                   "CubicAnisotropy(K1=5000000.0, u1=(0, 0, 1), u2=(0, 1, 0), "
                   "name=\"cubicanisotropy\")")
        assert repr(hamiltonian) == exp_str

    def test_getattr(self):
        hamiltonian = mm.Hamiltonian()
        for term in self.terms:
            hamiltonian += term

        assert isinstance(hamiltonian.exchange, mm.Exchange)
        assert hamiltonian.exchange.A == 1e-12

        assert isinstance(hamiltonian.zeeman, mm.Zeeman)
        assert hamiltonian.zeeman.H == (0, 0, 1.2e6)

        assert isinstance(hamiltonian.uniaxialanisotropy,
                          mm.UniaxialAnisotropy)
        assert hamiltonian.uniaxialanisotropy.K1 == 1e4
        assert hamiltonian.uniaxialanisotropy.u == (0, 1, 0)

        assert isinstance(hamiltonian.demag, mm.Demag)

        assert isinstance(hamiltonian.cubicanisotropy,
                          mm.CubicAnisotropy)
        assert hamiltonian.cubicanisotropy.K1 == 5e6
        assert hamiltonian.cubicanisotropy.u1 == (0, 0, 1)
        assert hamiltonian.cubicanisotropy.u2 == (0, 1, 0)

    def test_getattr_error(self):
        hamiltonian = self.exchange + self.zeeman

        with pytest.raises(AttributeError):
            demag = hamiltonian.demag

    def test_script(self):
        hamiltonian = mm.Hamiltonian()
        with pytest.raises(NotImplementedError):
            script = hamiltonian._script



import math

# Fundamental constants
mu0 = 4 * math.pi * 1e-7  # magnetic constant (N/A**2)
e = 1.6021766208e-19  # elementary charge (C)
me = 9.1093835611e-31  # electron mass (kg)
kB = 1.3806485279e-23  # Boltzmann constant (J/K)
h = 6.62607004081e-34  # Planck constant (Js)
g = 2.00231930436182  # Lande g-factor

# Derived constants
hbar = h/(2*math.pi)  # reduced Planck constant (Js)
gamma = g*e/(2*me)  # gyrotropic ratio (C/kg)
muB = gamma*hbar  # Bohr magneton (J/T)
gamma0 = gamma*mu0  # LLG precession term gamma (m/As)



import abc
import importlib


class Term(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __init__(self): pass  # pragma: no cover

    @property
    @abc.abstractmethod
    def _repr(self): pass  # pragma: no cover

    @property
    @abc.abstractmethod
    def _latex(self): pass  # pragma: no cover

    @property
    @abc.abstractmethod
    def _termsum_type(self): pass  # pragma: no cover

    def __repr__(self):
        """A representation method."""
        return self._repr

    def _repr_latex_(self):
        """A LaTeX representation method in Jupyter notebook."""
        return self._latex

    def __add__(self, other):
        """Addition operation."""
        module = importlib.__import__(self.__class__.__module__)
        termsum = getattr(module, self._termsum_type)()
        termsum._add(self)
        termsum._add(other)
        return termsum

    def __radd__(self, other):
        """Reverse addition for creating a list of energy objects."""
        other._add(self)
        return other

    @property
    def _script(self):
        """This method should be provided by the specific micromagnetic
        calculator"""
        raise NotImplementedError



import pytest
import numbers
import numpy as np
import micromagneticmodel as mm


class TestCubicAnisotropy:
    def setup(self):
        self.valid_args = [(1, (1, 0, 0), (0, 1, 0)),
                           (5e6, [1, 0, 1], [1, 1, 1]),
                           (-25.6e-3, (1, 0, 1), np.array([0, 0, -1])),
                           (1.5, [1e-6, 0, 0], [1e6, 1e6, 5e9])]
        self.invalid_args = [('1', (1, 0, 0), (0, 1, 0)),
                             (5e6, (-1, 0, -1), '(1, 1, 1)'),
                             (1e-3, (1, 0, 0), (0, 0, 0, 1)),
                             (5, (1, 0, 0), 5.0),
                             (-7e3, (0, 0, 1), ('1', 2e6, 0)),
                             ((1, 0, 0), (0, 0, 1), (0, 0, 1)),
                             (1, (5, 0), (0, 1, 0))]

    def test_init_valid_args(self):
        for K1, u1, u2 in self.valid_args:
            anisotropy = mm.CubicAnisotropy(K1=K1, u1=u1, u2=u2)
            assert anisotropy.K1 == K1
            assert isinstance(anisotropy.K1, numbers.Real)
            assert isinstance(anisotropy.u1, (tuple, list, np.ndarray))
            assert isinstance(anisotropy.u2, (tuple, list, np.ndarray))
            assert len(anisotropy.u1) == 3
            assert len(anisotropy.u2) == 3
            assert all([isinstance(i, numbers.Real) for i in anisotropy.u1])
            assert all([isinstance(i, numbers.Real) for i in anisotropy.u2])

    def test_init_invalid_args(self):
        for K1, u1, u2 in self.invalid_args:
            with pytest.raises(Exception):
                anisotropy = mm.CubicAnisotropy(K1=K1, u1=u1, u2=u2)

    def test_repr_latex(self):
        for K1, u1, u2 in self.valid_args:
            anisotropy = mm.CubicAnisotropy(K1=K1, u1=u1, u2=u2)
            latex = anisotropy._repr_latex_()

            # Assert some characteristics of LaTeX string.
            assert isinstance(latex, str)
            assert latex[0] == latex[-1] == '$'
            assert 'K_{1}' in latex
            assert '\mathbf{u}_{1}' in latex
            assert '\mathbf{u}_{2}' in latex
            assert '\mathbf{m}' in latex
            assert '^{2}' in latex
            assert '\cdot' in latex
            assert latex.count("\mathbf{u}_{1}") == 2
            assert latex.count("\mathbf{u}_{2}") == 2
            assert latex.count("\mathbf{u}_{3}") == 2

    def test_name(self):
        for K1, u1, u2 in self.valid_args:
            anisotropy = mm.CubicAnisotropy(K1=K1, u1=u1, u2=u2)
            assert anisotropy.name == 'cubicanisotropy'

    def test_repr(self):
        for K1, u1, u2 in self.valid_args:
            anisotropy = mm.CubicAnisotropy(K1=K1, u1=u1, u2=u2)
            exp_str = ("CubicAnisotropy(K1={}, u1={}, u2={}, "
                       "name=\"{}\")").format(K1, u1, u2, "cubicanisotropy")
            assert repr(anisotropy) == exp_str

        anisotropy = mm.CubicAnisotropy(1000, (0, 1, 0), (0, 0, 1),
                                        name="test_name")
        assert repr(anisotropy) == ("CubicAnisotropy(K1=1000, u1=(0, 1, 0), "
                                    "u2=(0, 0, 1), name=\"test_name\")")

    def test_script(self):
        for K1, u1, u2 in self.valid_args:
            anisotropy = mm.CubicAnisotropy(K1=K1, u1=u1, u2=u2)
            with pytest.raises(NotImplementedError):
                script = anisotropy._script



import joommfutil.typesystem as ts
from .dynamicsterm import DynamicsTerm


@ts.typesystem(u=ts.Vector(size=3),
               beta=ts.Scalar,
               name=ts.Name(const=True))
class STT(DynamicsTerm):
    _latex = ("$-(\mathbf{u} \cdot \\boldsymbol\\nabla)\mathbf{m} + "
              "\\beta\mathbf{m} \\times \\big[(\mathbf{u} \cdot "
              "\\boldsymbol\\nabla)\mathbf{m}\\big]$")

    def __init__(self, u, beta, name="stt"):
        """A spin transfer torque term.

        Args:
            u (RealVector): velocity vector
            beta (Real): non-adiabatic parameter

        """
        self.u = u
        self.beta = beta
        self.name = name

    @property
    def _repr(self):
        """A representation string property.

        Returns:
           A representation string.

        """
        return ("STT(u={}, beta={}, "
                "name=\"{}\")").format(self.u, self.beta, self.name)



import pytest
import pkg_resources
from .consts import mu0, e, me, kB, h, g, hbar, gamma, muB, gamma0
import micromagneticmodel.util  # to avoid import order conflicts
from .hamiltonian import EnergyProperties, EnergyTerm, Exchange, \
    UniaxialAnisotropy, CubicAnisotropy, Demag, Zeeman, DMI, Hamiltonian
from .dynamics import DynamicsTerm, Precession, \
    Damping, STT, Dynamics
from .drivers import Driver, MinDriver, TimeDriver, HysteresisDriver
from .system import System
from .data import Data


def test():
    return pytest.main(["-v", "--pyargs", "micromagneticmodel"])  # pragma: no cover

__version__ = pkg_resources.get_distribution(__name__).version
__dependencies__ = pkg_resources.require(__name__)



import joommfutil.typesystem as ts
from .energyterm import EnergyTerm


@ts.typesystem(H=ts.Vector(size=3),
               name=ts.Name(const=True))
class Zeeman(EnergyTerm):
    _latex = "$-\mu_{0}M_\\text{s} \mathbf{m} \cdot \mathbf{H}$"

    def __init__(self, H, name="zeeman"):
        """A Zeeman energy class.

        This method internally calls set_H method. Refer to its documentation.

        """
        self.H = H
        self.name = name

    @property
    def _repr(self):
        """A representation string property.

        Returns:
           A representation string.

        """
        return "Zeeman(H={}, name=\"{}\")".format(self.H, self.name)



import joommfutil.typesystem as ts
from .energyterm import EnergyTerm


@ts.typesystem(D=ts.Scalar,
               crystalclass=ts.InSet(allowed_values={"cnv", "t",
                                                     "o", "d2d",
                                                     "interfacial"}),
               name=ts.Name(constant=True))
class DMI(EnergyTerm):
    def __init__(self, D, crystalclass="t", name="dmi"):
        """A DMI energy class.

        Args:
            D (Real): DMI energy constant (J/m**2)
            
            crystalclass (string): Type of crystal class.
            Possible values are:
                "cnv", "t", "o", "d2d", "interfacial"

        """
        self.D = D
        self.crystalclass = crystalclass.lower()
        self.name = name

    @property
    def _latex(self):
        if self.crystalclass in ["t", "o"]:
            return ("$D \mathbf{m} \\cdot (\\nabla \\times \mathbf{m})$")
        elif self.crystalclass in ["cnv", "interfacial"]:
            return ("$D ( \mathbf{m} \\cdot \\nabla m_{z} "
                    "- m_{z} \\nabla \\cdot \mathbf{m} )$")
        else:
            return ("$D\mathbf{m} \\cdot \\left( \\frac{\\partial "
                    "\mathbf{m}}{\\partial x} \\times \hat{x} - "
                    "\\frac{\\partial \mathbf{m}}{\\partial y} "
                    "\\times \hat{y} \\right)$")

    @property
    def _repr(self):
        """A representation string property.

        Returns:
           A representation string.

        """
        return ("DMI(D={}, crystalclass=\"{}\", "
                "name=\"{}\")").format(self.D, self.crystalclass, self.name)
